#!/usr/bin/perl -w

# ustream controller
#	03.15.07 stark kfstark@starkconsulting.com, dtdb derik@gps.caltech.edu
#

package UStream;

=head1 NAME

UStream - Control ntripcaster

=head1 SYNOPSIS

use UStream;

=head1 DESCRIPTION

Control ntripcaster and real time streaming

All of the services created by this package rely on the daemontools package.

=cut

use strict;
use Data::Dumper;
use Config::Simple;
use File::Path;
use File::Basename;
use File::Spec::Functions;
use UStream::Log;
use UStream::Template;
use Net::Domain qw(hostname hostfqdn hostdomain);

our $VERSION = "0.06";
my $logger;
my $machine_name = hostname();
my $config_path = "";
my $service_path = "";

my $HOME = (getpwuid($>))[7];
my $USER = (getpwuid($>))[0];

my $USTREAMRC = $ENV{USTREAMRC};
#my $SHARC_RUN = "run.sharc";
#my $NTRIPC_RUN = "run.ntripcaster";
#my $XINETD_RUN = "run.xinetd";
#my $LOG_RUN = "run.log";
my $SHARC_RUN = basename($UStream::Template::files{SHARC});
my $NTRIPC_RUN = basename($UStream::Template::files{NTRIPC});
my $XINETD_RUN = basename($UStream::Template::files{XINETD});
my $LOG_RUN = basename($UStream::Template::files{LOG});

my $SITEINFO = "siteinfo.csv";
my $SHARC_CONFIG = "sharc.cfg";

# hardcode: svc has to be in path:
my $svc;
#my $old_PATH = $ENV{PATH};
#$ENV{PATH} = "/command:/sbin:/usr/sbin:/usr/local/sbin:/bin:/usr/bin:/usr/local/bin";
$svc = `which svc`; chomp($svc); 
#$ENV{PATH} = $old_PATH;

my $CONFIG_EXTENSION = ".conf";
my $ANTENNA_EXTENSION = ".antenna";
my $SITE_EXCLUDE = "ntrip|xinetd|ustream|NtripRTCMMC";

=head1 FILES

=over 4

=item ustreamrc

This package will attempt to read a config file from $HOME/.ustreamrc, /etc/ustreamrc, ./.ustreamrc in that order. If one of these files do not exist, a $HOME/.ustreamrc config file will be created with default values.

=item site config files

When a site is created, A config file will be placed in the I<ustream_config> directory with the site information. The config files are the 4 character site code with a I<.conf> extension.

=back

=cut

my @GLOBAL_CFG_FILES;
#::SEISMO changed search order of .ustreamrc files
push (@GLOBAL_CFG_FILES, "$ENV{USTREAMRC}");
push (@GLOBAL_CFG_FILES, "./.ustreamrc");
push (@GLOBAL_CFG_FILES, "$HOME/.ustreamrc");
push (@GLOBAL_CFG_FILES, "/etc/ustreamrc");


# TODO: fetch machine from env.
my $GLOBAL_DEFAULTS = {
	"base_path" => "$HOME/UStream/",
	"path.bin" => "bin/",
	"path.config" => "config/",
	"path.service" => "service/",		## Create server1/2.host subdirectories
	"path.xinetd" => "xinetd.d/",
	"path.log" => "logs/",
	"path.tmp" => "tmp/",
	"path.data" => "$HOME/UStream/data/",
	"path.data_sub" => "%Y/%j/",
	"path.html" => "www/html",
	"path.cgi" => "www/cgi-bin",
	"server1.datatype" => "binex",
	"server1.host"	=> "ntrip1",
	"server1.port"	=> 2101,
	"server1.ntrip_config" => "ntripcaster.conf",
	"server1.ntrip_sourcetable" => "sourcetable.dat",
	"server1.ntrip_agency" => "",
	"server2.datatype" => "rtcm",
	"server2.host"	=> "ntrip2",
	"server2.port"	=> 2101,
	"server2.ntrip_config" => "ntripcaster.conf",
	"server2.ntrip_sourcetable" => "sourcetable.dat",
	"server2.ntrip_agency" => "",
};

my $SITE_DEFAULTS = {
	"site.ipaddr"		=> "127.0.0.1",
	"site.ipport"		=> "2101",
	"site.port"             => "{site.ipaddr}:{site.ipport}",
	"site.mount"            => "",
	"site.ntriprelay"	=> "FALSE",
	"site.ntripmount"	=> "{site.code}_{site.datatype}",
	"site.ntripuser"	=> "",
	"site.ntrippass"	=> "",
	"site.code"		=> "TEST",
	"site.enabled"		=> "FALSE",
	"site.datatype"		=> "binex",
	"site.secondsinfile"	=> "3600",
	"site.data_filetemplate" => "{site.code}-%Y-%j-%H.{site.datatype}",
	"site.sample"		=> 1,
	"site.baud"		=> "",
	"site.sumfile"          => "",
	"site.login"            => "",
	"site.streamport"       => "A",
	"site.x"		=> 0,
	"site.y"		=> 0,
	"site.z"		=> 0,
	"site.lat"		=> 0,
	"site.long"		=> 0,
	"site.height"		=> 0,
	"site.refstaid"		=> 0,
	"site.receivertype"	=> "",
	"site.receivermodel"	=> "UNKNOWN",
	"site.antennamodel"	=> "UNKNOWN",
	"site.antennaserial"	=> "UNKNOWN",
	"site.antennafreq"	=> 0,
	"site.antennaheight"	=> 0.0083,
	"site.antennasetupid"	=> 0,
	"site.debug"            => 0,
	"site.debugfile"        => "-",
	"xinetd.enabled"	=> "FALSE",
	"xinetd.raw"		=> "FALSE",
	"xinetd.rtcm24"		=> "FALSE",
	"xinetd.rtcm30"		=> "FALSE",
	"ntrip.raw"		=> "FALSE",
	"ntrip.rtcm"		=> "FALSE",
	"ntrip.rtcmv24"		=> "FALSE",
	"ntrip.rtcmv30"		=> "FALSE",
};

my $SHARC_DEFAULTS = {
	"debug" => "{site.debug}",
	"debugfile" => "{site.debugfile}",
	"sumfile" => "{site.sumfile}",
	"login" => "{site.login}",
	"receivertype" => "{site.receivertype}",
#	"verifycode" => "{site.code}",
	"forcestream" => "1",
	"datatype" => "{site.datatype}",
	"datadir" => "{path.data}/{path.data_sub}",
	"port" => "{site.port}",
	"baud" => "{site.baud}",
	"stream" => "\"{site.sample},{site.streamport}\"",
	"ntrip" => "{site.ntripmount}:{encoder_password}\@{server_name}:{port}",
	"mount" => "{site.mount}",
	"file" => "\"{site.data_filetemplate},{site.secondsinfile},{site.sample}\"",
};

my $SHARC_RTCM = {
	"rtcm" =>  "set to getAntennaFile automatically\n",
	"rtcmv24" =>  "",
	"ntripv24" => "{site.code}_RTCM2:{encoder_password}\@{server_name}:{port}",
	"rtcmv30" =>  "",
	"ntripv30" => "{site.code}_RTCM3:{encoder_password}\@{server_name}:{port}",
};

my $NTRIP_CONFIG = {};

=head1 SUBROUTINES

=over 4

=item UStream::new

Create new UStream object.

=cut

sub new {
	my $class = shift;
	my $site = shift || undef;

	my $self = {};
	if (ref($site) eq "HASH") {
		$self = $site;
		$site = undef;
	}
		
	
        bless $self, $class;

	my $rc = $self->readUStreamConfig();
	if ($rc != 0) {
		print("Problem reading global config\n");
		return(undef);
	}

	$self->setSite($site) if ($site);
	$logger = new UStream::Log($self->getDir("path.log")) unless (defined($logger));

	$self;
}

=item UStream::setSite

Set site code for Object and load associated config file

=cut

sub setSite {
	my $self = shift;
	my $site = shift;

	$self->{site} = $site;

	undef($logger);
	$logger = new UStream::Log($self->getDir("path.log"),"${site}.log");

	$self->loadNtripConfig();
	# this call needs to be here, otherwise writeSiteConfig will overwrite with defaults:
	$self->loadSiteConfig();
}

=item UStream::readUStreamConfig

Read Global ustreamrc file

=cut

sub readUStreamConfig {
	my $self = shift;

	if (! $self->{GLOBAL_CONFIG}) {
		foreach (@GLOBAL_CFG_FILES) {
			if (-f $_) {
				$self->{GLOBAL_CONFIG} = $_;
				last;
			}
		}
	}

	$self->{GLOBAL_CONFIG} = $GLOBAL_CFG_FILES[0] unless ($self->{GLOBAL_CONFIG});

	my $modified = 0;
	$self->{CFG} = new Config::Simple();
	my $rc = $self->{CFG}->read("$self->{GLOBAL_CONFIG}");
	if (! $rc) {
		$self->{CFG}->syntax('ini');
		$logger = new UStream::Log("/tmp") unless(defined($logger));
		$logger->error("Unable to read config file\n");
		$logger->debug("Adding default values to the Global Config file\n");
		foreach (keys(%{$GLOBAL_DEFAULTS})) {
			$self->{CFG}->param($_, $GLOBAL_DEFAULTS->{$_});
			$modified = 1;
		}
	} else {
		# add in any pairs not defined in file but are defined at top:
		foreach (keys(%{$GLOBAL_DEFAULTS})) {
			if (! $self->{CFG}->param($_)){
				$self->{CFG}->param($_, $GLOBAL_DEFAULTS->{$_});
				$modified = 1;
			}
		}
	}
        $self->{CFG}->param("base_path",$self->{configure}) if (defined($self->{configure}) && -d $self->{configure});
	$self->{CFG}->save() if ($modified);

	return(0);
}

=item UStream::getPaths

Return hash of all paths in system set by ustreamrc config file

=cut

sub getPaths {
	my $self = shift;
	my %paths;
	$paths{base_path} = $self->{CFG}->param("base_path");
	foreach (keys(%{$GLOBAL_DEFAULTS})) {
		next if ($GLOBAL_DEFAULTS->{$_} =~ /%|}/);
		$paths{$1} = $self->getDir($_) if (/^path\.(\S+)$/);
	}
	return(%paths);

}

=item UStream::printPaths

print Paths to standard out for external scripts to get pathnames

useful for bash scripts

=cut

sub printPaths {
	my $self = shift;
	my %paths = $self->getPaths();
	foreach (keys(%paths)) {
		print "ustream_" . ${_} . "=" . $paths{$_}. "\n";
	}
}

=item UStream::isEnabled

return true if site is enabled, false otherwise

=cut

sub isEnabled {
	my $self = shift;

	if (!$self->siteExists()) {
		$logger->debug("$self->{site} does not exist. no action.\n");
		return 0;
	}

	if (! $self->{SITE_CFG}) {
		$logger->debug("No config file loaded\n");
		return 0;
	}

	return(1) if ($self->{SITE_CFG}->param("site.enabled") =~ /true/i);

	return 0;
}

=item UStream::createSiteConfig

create and write out site config for current site

=cut

sub createSiteConfig {
	my $self = shift;

	my $cfg_file = $self->getSiteConfigFile();

	$self->{SITE_CFG} = new Config::Simple();
	$self->{SITE_CFG}->syntax('ini');

	foreach (keys(%{$SITE_DEFAULTS})) {
		$self->{SITE_CFG}->param($_, $SITE_DEFAULTS->{$_});
	}
	$self->{SITE_CFG}->param("site.code", $self->{site});

	$self->{SITE_CFG}->write($cfg_file);
	$self->{SITE_CFG}->close();
	delete($self->{SITE_CFG});

	return($self->loadSiteConfig());
}

=item UStream::loadSiteConfig

Load site configuration file if it exists

=cut

sub loadSiteConfig {
	my $self = shift;

	if (defined($self->{SITE_CFG})) {
		delete($self->{SITE_CFG});
	}

	my $cfg_file = $self->getSiteConfigFile();
	$self->{SITE_CFG} = new Config::Simple("$cfg_file");


	return(0) if ($self->{SITE_CFG});
	$logger->debug("Unable to open Site config file\n");
	return(1);
}

=item UStream::loadNtripConfig

load ntrip config files to get basic info from files

=cut

# maybe generalize/combine these two subs.
sub loadNtripConfig {
	my $self = shift;

	my $ntrip_config_file = $self->getNtripConfigFile();
	$self->{NTRIP_CFG} = new Config::Simple("$ntrip_config_file");
	return(0) if ($self->{NTRIP_CFG});
	$logger->debug("Unable to open Ntrip config file\n");
	return(1);
}

=item UStream::modifySiteConfig

modify site parameter and return config object. Does not write config file

=cut

sub modifySiteConfig {
	my $self = shift;
	my $params = {@_};

	foreach (keys(%{$params})) {
		my $val = $self->{SITE_CFG}->param($_);
		if (! defined($val) ) {
			$logger->debug("Cannot modify unknow parameter $_\n");
			next;
		}
		$self->{SITE_CFG}->param($_, $params->{$_});
	}

	return(0);
}

=item UStream::writeSiteConfig

write the site configuration file to the config directory

=cut

sub writeSiteConfig {
	my $self = shift;

	# Add missing configuration options
	my %vars = $self->{SITE_CFG}->vars();
	foreach (keys(%{$SITE_DEFAULTS})) {
		next if (defined($vars{$_}));
		$self->{SITE_CFG}->param($_, $SITE_DEFAULTS->{$_});
	}
	# Remove extraneous configuration options
	%vars = $self->{SITE_CFG}->vars();
	foreach (keys(%vars)) {
		next if (defined($SITE_DEFAULTS->{$_}));
		$self->{SITE_CFG}->delete($_);
	}
	$self->{SITE_CFG}->write();
	delete($self->{SITE_CFG});
	return $self->loadSiteConfig();
}

=item UStream::createSite

Create a new site config file using configs passed in.

=cut

sub createSite {
	my $self = shift;
	my $configs = { @_ };

	my $event_success = "created";

	if ($self->siteExists()){
		$logger->debug("Error: Site already exists: $self->{site}\n");
		return(1);
	}

	$self->{siteconfigs} = $configs;

	my $rc = $self->createSiteConfig();
	if ($rc != 0) {
		$logger->debug("Unable to create Config file\n");
		return(1);
	}

	$rc = $self->modifySiteConfig(%{$configs});
	if ($rc != 0) {
		$logger->debug("Unable to modify configuration options\n");
		return(1);
	}

	$rc = $self->writeSiteConfig();

	return(0);
	### Log Success
}


=item UStream::updateSystem

Check running system against configuration and modify the running system to match.

This controls enabling/disabling of services.

=cut

sub updateSystem {
	my $self = shift;

	$logger->debug("Updating status of $self->{site}: ");
	$logger->debug("Entering updateSystem\n");
## Site is disabled or deleted, down and delete service directory
	my $cfg_file = $self->getSiteConfigFile();
	if ((! $self->isEnabled()) || (! -f $cfg_file)) {
		$self->downService();
		$self->deleteService();
		$logger->debug("DISABLED\n");
		return(0);
	}

## SITE exists and is enabled ##

	$logger->debug("Site is enabled\n");
	my $old_cfg = $self->loadSharcConfig();
	my $new_cfg = $self->makeSharcConfig();
	if ($self->compareSimpleConfigObjects($old_cfg,$new_cfg) != 0) {
		$logger->debug("Creating new sharc config and restarting service\n");
		$self->downService();
		$self->deleteService();
		$self->createService();
		$new_cfg->write($self->getSharcConfigFile());
		if ($self->{SITE_CFG}->param("ntrip.rtcm") =~ /true/i){
		    $self->addRTCMConfigs($self->getSharcConfigFile());
		    $self->makeAntennaFile();
		}
		$self->makeXinetdFile();
		$self->upService();
		$logger->debug("ENABLED\n");
	}
	return(0);

}

=item UStream::createNtripCasterService

Create the ntripcaster service directory for daemontools

=cut

sub createNtripCasterService {
	my $self = shift;
	my $svc_dir = catfile($self->getDir("path.service"),"ntripcaster");
	$self->createService($svc_dir,$NTRIPC_RUN);
	$self->writeTemplateFile($svc_dir, "ntripcaster.conf", "ntripcaster.conf");
	$self->upService($svc_dir);
}

=item UStream::deleteNtripCasterService

Down the ntripcaster service and delete the service directory.

=cut

sub deleteNtripCasterService {
	my $self = shift;
	my $svc_dir = catfile($self->getDir("path.service"),"ntripcaster");
	$self->downService($svc_dir);
	$self->deleteService($svc_dir);
}

=item UStream::createClientService

Create the client service directory for daemontools

=cut

sub createClientService {
	my $self = shift;
	my $progname = shift;
	my $svc_dir = catfile($self->getDir("path.service"),$progname);
	$self->createService($svc_dir,"run.${progname}");
	$self->writeTemplateFile($svc_dir, "${progname}.conf", "${progname}.conf");
	$self->upService($svc_dir);
}

=item UStream::deleteClientService

Down the client service and delete the service directory.

=cut

sub deleteClientService {
	my $self = shift;
	my $progname = shift;
	my $svc_dir = catfile($self->getDir("path.service"),"$progname");
	$self->downService($svc_dir);
	$self->deleteService($svc_dir);
}

=item UStream::createXinetdService

Create the ntripcaster service directory for daemontools

=cut

sub createXinetdService {
	my $self = shift;
	my $svc_dir = catfile($self->getDir("path.service"),"xinetd");
	$self->createService($svc_dir,$XINETD_RUN,"none");
	$self->writeTemplateFile($svc_dir, "xinetd.conf", "xinetd.conf");
	$self->upService($svc_dir);

}

=item UStream::deleteXinetdService

Down the ntripcaster service and delete the service directory.

=cut

sub deleteXinetdService {
	my $self = shift;
	my $svc_dir = catfile($self->getDir("path.service"),"xinetd");
	$self->downService($svc_dir);
	$self->deleteService($svc_dir);
}

=item UStream::upService

Use the svc command to up an existing service. return undef if service does not exist

=cut

sub upService {
	my $self = shift;
	my $svc_dir = shift ||  $self->getSiteServiceDir();
	my $rc;

	my $log_dir = catfile($svc_dir, "log");

	return(undef) if (! -d $svc_dir);

	## Add down files
	my $log_down = catfile($svc_dir,"down"); 
	my $svc_down = catfile($log_dir,"down"); 
	unlink($log_down) if (-f $log_down);
	unlink($svc_down) if (-f $svc_down);

	sleep(2);
	$self->my_system("$svc","-u",$svc_dir,$log_dir);
	sleep(1);
	$self->my_system("$svc","-u",$svc_dir,$log_dir);
	return(0);

}

=item UStream::downService

Use "down" file and svc command to down service.

=cut

sub downService {
	my $self = shift;
	my $svc_dir = shift ||  $self->getSiteServiceDir();

	my $log_dir = catfile($svc_dir, "log");

	return(0) if (! -d $svc_dir);

	## Add down files
	my $log_down = catfile($svc_dir,"down"); 
	my $svc_down = catfile($log_dir,"down"); 
	open(F,">$log_down") && close(F);
	open(F,">$svc_down") && close(F);

	$self->my_system("$svc","-d",$svc_dir,$log_dir);
	sleep(1);
	$self->my_system("$svc","-d",$svc_dir,$log_dir);
	return(0);

}

=item UStream::makeHiddenDir

Move an existing directory to a hidden directory. This is necessary to properly shutdown a service running under daemontools.

=cut

sub makeHiddenDir {
	my $self = shift;
	my $orig_dir = shift;

	return(undef) unless (-d $orig_dir);
	my @dirs = File::Spec->splitdir($orig_dir);
	my $i = $#dirs;
	return($orig_dir) if ($dirs[$i] =~ /^\./); 
	$dirs[$i] = "." . $dirs[$i];
	my $new_dir = catdir(@dirs);
	rmtree($new_dir) if (-d $new_dir);
	rename($orig_dir, $new_dir) || return(undef);
	return($new_dir);
}


=item UStream::deleteService

Delete a service being run by daemontools.

=cut

sub deleteService {
	my $self = shift;
	my $svc_dir = shift ||  $self->getSiteServiceDir();

	my $log_dir = catfile($svc_dir, "log");

	return(0) if (! -d $svc_dir);

	for (my $i = 0; $i < 10 && (-d $svc_dir); $i++) {
		$logger->debug("Attempt $i to remove Service\n");
		$self->downService($svc_dir);
		my $new_dir = $self->makeHiddenDir($svc_dir);
		my $new_log_dir = catfile($new_dir,"log");
		$self->my_system("$svc","-dx",$new_dir,$new_log_dir);
		sleep(1);
		$self->my_system("$svc","-kx",$new_dir,$new_log_dir);
		sleep(1);
		rmtree($new_dir);
		sleep(1);
	}
	return(0);
}

=item UStream::createService

create a service under daemontools

=cut

sub createService {
	my $self = shift;
	my $svc_dir = shift ||  $self->getSiteServiceDir();
	my $runfile = shift || $SHARC_RUN;
	my $runlog  = shift || $LOG_RUN;

## do not create if it already exists
	return(1) if (-d $svc_dir);

	eval { mkpath($svc_dir) };
	if ($@) {
		$logger->warn("Couldn't create $svc_dir: $@");
		return(1);
	}
	if ($runlog ne "none") {
		my $log_dir = catfile($svc_dir, "log");
		eval { mkpath($log_dir) };
		if ($@) {
			$logger->warn("Couldn't create $log_dir: $@");
			return(1);
		}
		$self->writeTemplateFile($log_dir, $runlog, "run") if ($runlog ne "none");
	}

	my $mode = (stat($svc_dir))[2];
	$mode &=  07777; $mode |= 1000;
	chmod $mode, $svc_dir;

	$self->downService($svc_dir);

	$self->writeTemplateFile($svc_dir, $runfile, "run");

}

=item UStream::writeTemplateFile

Write out a file based on a template. Uses metaReplace to modify fields in file.

=cut

sub writeTemplateFile {
	my $self = shift;
	my $out_dir = shift;
	my $runfile = shift;
	my $outfile = shift;


	if (!-d $out_dir){
		$logger->debug("something wrong $out_dir does NOT exist\n");
		return(1);
	}

	my $in_file = catfile($self->getDir("path.config"), $runfile);
	my $out_file = catfile($out_dir,$outfile);

	return(1) unless (-e $in_file);

	open(OUT, ">$out_file") or return(1); 
	open(IN,"<$in_file") or return(1);
	while(<IN>) {
		print OUT $self->metaReplace($_);
	}
	close(IN);
	close(OUT);
	
	chmod 0755, $out_file;

	return(0);
}


=item UStream::loadSharcConfig

Load existing sharc config for comparison

=cut

sub loadSharcConfig {
	my $self = shift;

        my $sharc_cfg_file = $self->getSharcConfigFile();

	return(undef) unless (-e $sharc_cfg_file);
	my $sharc_cfg = new Config::Simple();
	my $rc = $sharc_cfg->read($sharc_cfg_file);
	if (!$rc) {
		$logger->debug("some problem creating sharc_cfg from $sharc_cfg_file. rc: $rc \n");
		return(undef);
	}
	return($sharc_cfg);
}

=item UStream::addRTCMConfigs

Add RTCM and NTRIP arguments to sharc config file if they are enabled

=cut

sub addRTCMConfigs {
	my $self = shift;
	my $filename = shift;

	my $rtcm24 = 0; my $rtcm30 = 0;
	return(0) unless ($self->{SITE_CFG}->param("ntrip.rtcm") =~ /true/i);
	$rtcm24 = 1 if ($self->{SITE_CFG}->param("ntrip.rtcmv24") =~ /true/i);
	$rtcm30 = 1 if ($self->{SITE_CFG}->param("ntrip.rtcmv30") =~ /true/i);
	return(0) unless ($rtcm24 + $rtcm30);

	return (undef) if (!-e $filename);
	open(RTCM,">> $filename");
	
	my $out = "";
	$out .= "rtcm " . $self->getAntennaFile() . "\n";
	if ($rtcm24) {
		$out .= "rtcmv24 1\n";
		$out .= "ntrip " . $self->metaReplace($SHARC_RTCM->{ntripv24}) . "\n";
	}
	if ($rtcm30) {
		$out .= "rtcmv30 1\n";
		$out .= "ntrip " . $self->metaReplace($SHARC_RTCM->{ntripv30}) . "\n";
	}

	print RTCM $out;

	close(RTCM);
	return(0);

}

=item UStream::makeSharcConfig

Create the sharc config file to be written to the site's service directory

=cut

sub makeSharcConfig {
	my $self = shift;

	my $sharc_cfg = new Config::Simple();
	$sharc_cfg->syntax('simple');
	# first fill with defaults
        foreach (keys(%{$SHARC_DEFAULTS})) {
                $sharc_cfg->param($_, $SHARC_DEFAULTS->{$_});
        }

	# then overlay any values we can derive from main config file:
	my ($tmp,@tmp);
        foreach (keys(%{$SHARC_DEFAULTS})) {
		$tmp = $sharc_cfg->param($_);
		if (ref($tmp) eq "ARRAY"){
			my @new_vals;
			foreach my $t(@{$tmp}){
				$t = $self->metaReplace($t);
				push @new_vals, $t;
			}
			$sharc_cfg->param($_,\@new_vals);
		}
		else{
			$tmp = $self->metaReplace($tmp);
			$sharc_cfg->param($_,$tmp);
		}
	}

	if ($self->{SITE_CFG}->param("ntrip.raw") =~ /false/i){
		$sharc_cfg->delete("ntrip");
	}
	#::SEISMO if port is still empty set it to default value
        if ($sharc_cfg->param("port") eq ""){
             $tmp = $self->metaReplace("{site.ipaddr}:{site.ipport}");
             $sharc_cfg->param("port",$tmp);
        }
	if ($self->{SITE_CFG}->param("site.baud") eq ""){
		$sharc_cfg->delete("baud");
	}
	if ($self->{SITE_CFG}->param("site.sumfile") eq ""){
		$sharc_cfg->delete("sumfile");
	}
	if ($self->{SITE_CFG}->param("site.login") eq ""){
		$sharc_cfg->delete("login");
	}
	if ($self->{SITE_CFG}->param("site.receivertype") eq ""){
		$sharc_cfg->delete("receivertype");
	}
	if ($self->{SITE_CFG}->param("site.ntriprelay") eq ""){
		$sharc_cfg->delete("mount");
	}
	if ($self->{SITE_CFG}->param("site.ntriprelay") =~ /false/i){
		$sharc_cfg->delete("mount");
	}
	if ($self->{SITE_CFG}->param("site.data_filetemplate") eq ""){
		$sharc_cfg->delete("file");
	}
	return($sharc_cfg);
}

=item UStream::makeAntennaFile

Make the antenna file from the positions in the site config file. This will be used for the RTCM streaming.

=cut

sub getSiteInfoEntry {
	my $self = shift;

	my $ret = {};
	my $code = $self->{site};
	my $infofile = catfile($self->getDir("path.config"),$SITEINFO);
	return(undef) unless (-f $infofile);
	open(INFO,"<$infofile") || return(undef);
	my $found = 0;

	while (<INFO>) {
		next unless (/^$code/i);
		my @f = split(",");
		next if ($#f < 4);
		$ret = {"x"  => $f[1], 
			"y"  => $f[2],
			"z"  => $f[3],
			"hi" => $f[4] };
		$ret->{"ant_model"} = $f[5] if ($#f >= 4);
		$ret->{"ant_serial"} = $f[6] if ($#f >= 6);
		$found = 1;
		last;
	}
	return(undef) unless ($found);
	my $use_conf = 0;
	$use_conf ++ if ($self->{SITE_CFG}->param("site.x") == 0);
	$use_conf ++ if ($self->{SITE_CFG}->param("site.y") == 0);
	$use_conf ++ if ($self->{SITE_CFG}->param("site.z") == 0);
	if (! $use_conf == 3) {
		$ret->{"x"} = $self->{SITE_CFG}->param("site.x");
		$ret->{"y"} = $self->{SITE_CFG}->param("site.y");
		$ret->{"z"} = $self->{SITE_CFG}->param("site.z");
		$ret->{"hi"} = $self->{SITE_CFG}->param("site.antennaheight");
	}
	$ret->{"ant_model"} =  $self->{SITE_CFG}->param("site.antennamodel")
		unless ($self->{SITE_CFG}->param("site.antennamodel") eq "UNKNOWN");
	$ret->{"ant_serial"} =  $self->{SITE_CFG}->param("site.antennaserial")
		unless ($self->{SITE_CFG}->param("site.antennaserial") eq "UNKNOWN");

	return($ret);
		
}
=item UStream::makeAntennaFile

Make the antenna file from the positions in the site config file. This will be used for the RTCM streaming.

=cut

sub makeAntennaFile {
	my $self = shift;
	my $rtcm_ver = shift || 4;

	my $x = $self->{SITE_CFG}->param("site.x");
	my $y = $self->{SITE_CFG}->param("site.y");
	my $z = $self->{SITE_CFG}->param("site.z");
	my $siteinfo = $self->getSiteInfoEntry();
	my $xtmp = $x * 10000;
	my $ytmp = $y * 10000;
	my $ztmp = $z * 10000;
	my $xcm = int($xtmp / 100) / 100.0;
	my $ycm = int($ytmp / 100) / 100.0;
	my $zcm = int($ztmp / 100) / 100.0;
	my $xdelta = int(abs($xtmp) % 100) / 100;
	my $ydelta = int(abs($ytmp) % 100) / 100;
	my $zdelta = int(abs($ztmp) % 100) / 100;
	if ($xdelta > .5) {
		$xdelta = 1 - $xdelta;
		$xcm += ($xcm > 0) ? .01 : -.01;
	} else {
		$xdelta *= -1 if ($xcm < 0);
	}
	if ($ydelta > .5) {
		$ydelta = 1 - $ydelta;
		$ycm += ($ycm > 0) ? .01 : -.01;
	} else {
		$ydelta *= -1 if ($ycm < 0);
	}
	if ($zdelta > .5) {
		$zdelta = 1 - $zdelta;
		$zcm += ($zcm > 0) ? .01 : -.01;
	} else {
		$zdelta *= -1 if ($zcm < 0);
	}
			
	my $output_file = $self->getAntennaFile();
	open(OUT,">$output_file");
	my $antfreq      = $self->{SITE_CFG}->param("site.antennafreq");
	my $antmodel     = $self->{SITE_CFG}->param("site.antennamodel");
	my $antserial    = $self->{SITE_CFG}->param("site.antennaserial");
	my $antheight    = $self->{SITE_CFG}->param("site.antennaheight");
	my $refstaid     = $self->{SITE_CFG}->param("site.refstaid") % 1000;
	my $antsetupid   = $self->{SITE_CFG}->param("site.antennasetupid");
	my $antmodellen  = length($antmodel);
	my $antseriallen = length($antserial);
			
print OUT <<EOF2;
$rtcm_ver	$refstaid	$antfreq
H	23	$refstaid	0.0	0	5	0
I	1	1	$antsetupid	$antmodellen	$antseriallen
ID	$antmodel
IS	$antserial
H	24	$refstaid	0.0	0	6	0
Z	1	1	$x	$y	$z	$antheight
H	3	$refstaid	0.0	0	4	0
R	$xcm	$ycm	$zcm
H	22	$refstaid	0.0	$antsetupid	1	0
M1	$xdelta	$ydelta	$zdelta
H	1005	$refstaid	0.0	0	0	0
5	0	0	0	0	0
51	0	0	$x	$y	$z
H	1006	$refstaid	0.0	0	0	0
5	1	1	$antsetupid	$antmodellen	$antseriallen
51	1	1	$x	$y	$z
62	$antheight
H	1007	$refstaid	0.0	0	0	0
7	$antsetupid	$antmodel
H	1008	$refstaid	0.0	0	0	0
7	$antsetupid	$antmodel
81	$antserial
EOF2
	close(OUT);

}

=item UStream::makeAntennaFile

Make the antenna file from the positions in the site config file. This will be used for the RTCM streaming.

=cut

sub makeXinetdFile {
	my $self = shift;

	return(0) unless ($self->{SITE_CFG}->param("xinetd.enabled") =~ /true/i);

	my $svc_dir = catfile($self->getDir("path.service"),"xinetd");

	if (($self->{SITE_CFG}->param("site.refstaid") < 1000) && ($self->{SITE_CFG}->param("site.refstaid") > 99)) {
		$self->writeTemplateFile($self->getDir("path.xinetd"), "xinetd_site.conf", $self->{site});
	}

	## force reread of site directory after new configuration file installed
	my $pid_file = catfile($svc_dir,"xinetd.pid");
	open(FILEHANDLE, "<$pid_file") || return(1);
	my $pid = do { local $/; <FILEHANDLE> };
	kill 1, $pid;
}

=item UStream::metaReplace

Replace meta strings in passed string. Meta strings are defined from config files.

=cut

sub metaReplace {
	my $self = shift;
	my $val = shift;

	my $tmp;
	if (defined($self->{site})){
		$tmp = $self->{site};
		$val =~ s/{site.code}/$tmp/g;
	}

	my (%vars1, %vars2, %vars3);
	my %vars;
	%vars1 = $self->{CFG}->vars() if (defined($self->{CFG}));
	%vars2 = $self->{SITE_CFG}->vars() if (defined($self->{SITE_CFG}));
	%vars3 = $self->{NTRIP_CFG}->vars() if (defined($self->{NTRIP_CFG}));
	foreach (keys(%vars1)) {
		$vars{$_} = $vars1{$_};
	}
	foreach (keys(%vars2)) {
		$vars{$_} = $vars2{$_};
	}
	foreach (keys(%vars3)) {
		$vars{$_} = $vars3{$_};
	}
## Weird one
	$vars{"base_path"}        = $self->{CFG}->param("base_path");
	$vars{"ustream_user"}     = $USER;
## Take care of service "disable =" by reversing meaning
	$vars{"xinetd.raw"}    = ($vars{"xinetd.raw"}    =~ /true/i) ? "no" : "yes";
	$vars{"xinetd.rtcm24"} = ($vars{"xinetd.rtcm24"} =~ /true/i) ? "no" : "yes";
	$vars{"xinetd.rtcm30"} = ($vars{"xinetd.rtcm30"} =~ /true/i) ? "no" : "yes";
	for (my $i = 0; $i < 3; $i++) {
		while (my ($in, $out) = each(%vars)) {
			if ($in =~ /^path\.[a-zA-Z]+$/) {
				$out = $self->getDir($in);
			}
			$in = '{' . $in . '}';
#			my $out = $vars{$_};
			$val =~ s/$in/$out/g;
		}
	}

	return $val;
}


=item UStream::deleteSite

Delete a site's configuration file

=cut

sub deleteSite {
	my $self = shift;

	if (! $self->siteExists()) {
		$logger->debug("$self->{site} does not exist. no action.\n");
		return 0;
	}

	my $cfg_file = $self->getSiteConfigFile();
	if (! unlink $cfg_file) {
		$logger->("Unable to delete file: $cfg_file\n");
		return(1);
	}

	return 0;
}

=item UStream::disableSite

Set site to be disabled.

=cut

sub disableSite {
	shift->enableSite(0);
}

=item UStream::enableSite

Set site to be enabled

=cut

sub enableSite {
	my $self = shift;
	my $enable = shift;

	$enable = 1 if (!defined($enable));

	if (! $self->{site}) {
		$logger->debug("Sitecode not defined\n");
		return(1);
	}

	if (! $self->siteExists()){
		$logger->debug("$self->{site} does not exist. no action.\n");
		return 1;
	}	

	$self->loadSiteConfig();
	$self->modifySiteConfig("site.enabled" => ($enable) ? "TRUE" : "FALSE");
	$self->writeSiteConfig();

	##LOG IT
	return(0);
}

=item UStream::compareSimpleConfigObjects

Compare Config Objects to determine if a site has been changed to update running system if necessary

=cut

sub compareSimpleConfigObjects {
	my $self = shift;
	my $old = shift || return(1);
	my $new = shift || return(1);

	return(1) unless (ref($old));
	return(1) unless (ref($new));

	my @old = map { (/^\s*#/) ? "" : $_ } sort(split("\n",$old->as_string()));
	my @new = map { (/^\s*#/) ? "" : $_ } sort(split("\n",$new->as_string()));

	return(1) unless ($#old == $#new);

	for (my $i = 0; $i < $#old; $i++) {
		next if ($old[$i] eq $new[$i]);

		my ($old1, $old2) = split(/ /,$old[$i]);
		my ($new1, $new2) = split(/ /,$new[$i]);
		$new2 = $1 if ($new2 =~ /^"(.*)"$/);
		return(1) unless ($old1 eq $new1);
		return(1) unless ($old2 eq $new2);
	}
	return(0);

}

=item UStream::getDir

Return a directory path based on the config file

=cut

sub getDir {
	my $self = shift;
	my $param = shift;

	my $dir = $self->{CFG}->param($param);
	return($dir) if ($dir =~ /^\//);

	return catfile($self->{CFG}->param("base_path"), $dir);
	
}


=item UStream::getSiteConfigFile

Get the path to the site configuration file

=cut

sub getSiteConfigFile {
	my $self = shift;
	return catfile($self->getDir("path.config"),$self->{site} . $CONFIG_EXTENSION);
}

=item UStream::getSiteList

get a list of all created sites in the system

=cut

sub getSiteList {
	my $self = shift;
	my $exclude = shift || $SITE_EXCLUDE;
	opendir(DIR, $self->getDir("path.config"));

	#my @tmp_sites = grep { /^\S{4}$CONFIG_EXTENSION$/ } readdir(DIR);
	#my @tmp_codes = map {  $1 if (/(\S{4})$CONFIG_EXTENSION$/) } @tmp_sites;
	my @tmp_sites = grep { $_ !~ m/$exclude/i and /^\S+$CONFIG_EXTENSION$/ } readdir(DIR);
	my @tmp_codes = map {  $1 if (/(\S+)$CONFIG_EXTENSION$/) } @tmp_sites;

	return(@tmp_codes);
}

=item UStream::getSharcConfigFile

get a path to the sharc config file for the given site

=cut

sub getSharcConfigFile {
	my $self = shift;
	return catfile($self->getSiteServiceDir(),$SHARC_CONFIG);
}

=item UStream::getAntennaFile

get a path to the Antenna File to be used by sharc

=cut

sub getAntennaFile {
	my $self = shift;
	return catfile($self->getSiteServiceDir(),$self->{site} . $ANTENNA_EXTENSION);
}

=item UStream::getNtripConfigFile

get path to ntrip config file

=cut

sub getNtripConfigFile {
	my $self = shift;
	my @files = (
		catfile($self->getDir("path.config"),$self->{CFG}->param("server1.ntrip_config")) ,
		catfile($self->getDir("path.config"),$self->{CFG}->param("server2.ntrip_config")) ,
		catfile("/etc/ntripcaster",$self->{CFG}->param("server1.ntrip_config")) ,
		catfile("/etc/ntripcaster",$self->{CFG}->param("server2.ntrip_config")) ,
		catfile("/usr/local/etc/ntripcaster",$self->{CFG}->param("server1.ntrip_config")) ,
		catfile("/usr/local/etc/ntripcaster",$self->{CFG}->param("server2.ntrip_config")) ,
		catfile("/usr/local/ntripcaster/etc",$self->{CFG}->param("server1.ntrip_config")) ,
		catfile("/usr/local/ntripcaster/etc",$self->{CFG}->param("server2.ntrip_config")) ,
	);
	foreach (@files) {
		$_ = $self->metaReplace($_);
		return($_) if (-e $_);
	}
	wantarray ? return(@files) : return($files[0]);
}


=item UStream::getSiteServiceDir

 returns service dir for $site
	if already exists, return it.
	if doesn't exist, return path where it should be created

=cut

sub getSiteServiceDir {
	my $self = shift;

	return catfile($self->getDir("path.service"),$self->{site});


	my @dirs = (
		catfile($self->getDir("path.service"),$self->{CFG}->param("server1.host")),
		catfile($self->getDir("path.service"),$self->{CFG}->param("server2.host"))
	);

	foreach (@dirs){
		my $t = catfile($_,$self->{site});
		return($t) if (-d $t);
	}


	my (@sub_dirs);
	my $least_num = 999;
	my $least_num_dir = $dirs[0];
	foreach my $dir (@dirs) {
		$dir = $self->metaReplace($dir);
		@sub_dirs = <$dir*>;
		my $sub_dir_count = scalar(@sub_dirs);
		if ($sub_dir_count < $least_num){
			$least_num = scalar(@sub_dirs);
			$least_num_dir = $dir;
		}
	}

	my $return_dir = catfile($least_num_dir,$self->{site});
	wantarray ? return(@dirs) : return($return_dir);
}

=item UStream::siteExists

Return true if site exists, false otherwise

=cut

sub siteExists {
	my $self = shift;

	my $cfg_file = $self->getSiteConfigFile();
	return(0) unless (-f $cfg_file);
	return(1);
}

=item UStream::my_system

run system command safely

=cut

sub my_system {
	my $self=shift;
	my @args = @_;

	open OLDERR, ">&", \*STDERR;
	print OLDERR "";
	open(STDERR, '>/dev/null');
	system(@args);
	open(STDERR, ">&OLDERR"); # restore STDERR

}

=item UStream::writeTemplates

Write out template files

=cut

sub writeTemplates {
	my $self = shift;
	foreach my $tmpl (keys(%UStream::Template::files)) {
		my $file = $self->metaReplace($UStream::Template::files{$tmpl});
		my $contents = $UStream::Template::contents{$tmpl};
		open(OUT,">$file") || return(undef);
		print OUT $contents;
		close(OUT);
	}
}

sub getSiteConfigHash { my $self = shift; return($self->{SITE_CFG}->vars()); }

sub getLogger { $logger }
1;

__END__

=back

=head1 AUTHOR

Keith F. Stark (Stark Consulting LLC)

=head1 SEE ALSO

L<ustreamrc>, L<ustream-logger>, L<ustream.cgi>, L<UStream>,  L<UStream::CGI>, L<UStream::Log>, L<UStream::Template>

=cut
