#!/usr/bin/perl
# ustream_cgi.pm - show, add, modify, delete site info

use strict;
package UStream::CGI;

=head1 NAME

UStream::CGI - Web Server interface UStream

=head1 SYNOPSIS

use UStream::CGI;

=head1 DESCRIPTION

This handle the entire web interface for ustream including building all forms and reporting site status. 

Uses L<CGI::FormBuilder> to build all of the forms

=cut

use CGI;
use CGI::FormBuilder;
use Data::Dumper;
use UStream::Log;
use File::Basename;
use File::Spec::Functions;
use Fcntl;
use GDBM_File;
use Switch;

my $calls = {
	all	=> \&mode_all,
	show	=> \&mode_show,
	add	=> \&mode_add,
	modify	=> \&mode_modify,
	delete	=> \&mode_delete,
	log	=> \&mode_log,
	system	=> \&mode_system,
	start	=> \&mode_start,
	status	=> \&mode_status,
	info	=> \&mode_info,
	stop	=> \&mode_stop,
	restart	=> \&mode_restart,
	};

=head1 SUBROUTINES

=over 4

=item UStream::CGI::new

Create a new CGI object and repond to command line

=cut

sub new {
	my $class = shift;
	my $query = shift;
	my $self = {};

	my $mode = $query->param('mode');
	$mode = "status" unless(defined($mode));
	$self->{query} = $query;
	$self->{ustream} = new UStream();
	$self->{script_name} = $self->{query}->script_name();
	$self->{content} = "";

	bless $self, $class;

	if (defined($calls->{$mode})) {
		my $t = $calls->{$mode};
		$self->$t();
	} else {
		$self->add_message("UNKNOW MODE $mode");
	}

	$self;
}

=item UStream::CGI::add_content

Append string to content hash to be returned from object

=cut

sub add_content {
	my $self = shift;
	$self->{content} .= shift;
	$self->{content} .=  "\n";
}

=item UStream::CGI::add_message

Append warning string to the content hash wrapped in div to identify error

=cut

sub add_message {
	my $self = shift;
	my $mess = shift;
	$self->add_content("<span id=\"error\">$mess</span><p/>");
}

=item UStream::CGI::get_html

return HTML content

=cut

sub get_html {
	my $self = shift;
	return $self->{content};
}

=item UStream::CGI::mode_all

Show all sites in system and buttons to get more info

=cut

sub mode_all {
	my $self = shift;
	my @sites = $self->{ustream}->getSiteList();

	my $comment;
	$self->add_content("<table id=\"sitelist\">");
	foreach my $site (sort(@sites)) {
		$site = uc($site);
		$comment .= "$site\n";
		$self->add_content("<tr><td>");
		$self->add_content("<a href=\"$self->{script_name}?mode=show&site=$site\">$site</a>");
		$self->add_content("</td><td>");
		$self->buttons($site);
		$self->add_content("</td></tr>");
	}
	$self->add_content("</table>");
	$self->add_content("<!-- all\n$comment\n -->");

}

=item UStream::CGI::buttons

return HTML to output buttons for a site

=cut

sub buttons {
	my $self = shift;
	my $site = shift;
	$site = uc($site);
	$self->{ustream}->setSite($site);
	my %vars = $self->{ustream}->getSiteConfigHash();
	my $ip = $vars{"site.ipaddr"};
	$self->add_content("<span id=\"buttons\">");
	$self->add_content("<a href=\"$self->{script_name}?mode=modify&site=$site\">Modify</a>");
	$self->add_content("<a href=\"$self->{script_name}?mode=delete&site=$site\">Delete</a>");
	$self->add_content("<a href=\"$self->{script_name}?mode=log&site=$site\">Logs</a>");
	$self->add_content("<a href=\"$self->{script_name}?mode=info&site=$site\">Info</a>");
	$self->add_content("<a href=\"http://$ip/\">Receiver</a>");
	$self->add_content("</span>");

}

=item UStream::CGI::mode_show

Show information about a site

=cut

sub mode_show {
	my $self = shift;
	# Show existing site
	my @sites = $self->{query}->param('site');
	if ($#sites != 0) {
		$self->add_message("ERROR: No site to show");
		$self->mode_all();
		return(0);
	}
	my $site = uc($sites[0]);
	$self->{ustream}->setSite($site);
	if (! $self->{ustream}->siteExists()) {
		$self->add_message("Site $site doesn't exist");
		$self->mode_all();
		return(0);
	} else {
		$self->add_content("<table id=\"sitesettings\">");
		$self->add_content("<th>$site Settings:</th>");
		$self->add_content("<th>");
		$self->buttons($site);
		$self->add_content("</th>");
		my %vars = $self->{ustream}->{SITE_CFG}->vars();
		my $comment;
		foreach (sort(keys(%vars))) {
			$self->add_content("<tr>");
			$self->add_content("<td>$_</td>");
			$self->add_content("<td>$vars{$_}</td>");
			$self->add_content("</tr>");
			$comment .= "$_ = $vars{$_}\n";
		}
		$self->add_content("</table>");
		my $sinfo = $self->{ustream}->getSiteInfoEntry();
		foreach (sort(keys(%{$sinfo}))) {
			$comment .= "siteinfo." . $_ . " = " . $sinfo->{$_} . "\n";
		}
		$self->add_content("<!-- show\n$comment\n -->");
		
	}
	$self->mode_all();
}

=item UStream::CGI::mode_add

output form to add new site

=cut

sub mode_add {
	my $self = shift;
	# New article
	my $form = CGI::FormBuilder->new(
		params => $self->{query},           # params from CGI.pm 
		submit => [qw/Add Cancel/],
		keepextras => 1
	);
	$form->field(name => "site", label => "Four Character Site Code", size=>5, maxlength=>4);
	$form->field(name => "ipaddress", label => "IP Address", size=>20);
	$form->field(name => "ipport", label => "IP Port", size=>6, maxlength=>5);
	$form->field(name => "enablesite", label => "", options => ["Enable Site"]);
	$form->field(name => "enablebinex", label => "", options => ["Enable Binex Streaming"]);
	$form->field(name => "enablertcm", label => "", options => ["Enable RTCM Streaming"]);

	if (($form->submitted eq "Add") && $form->validate) {
		my $site = $self->{query}->param("site");
		$site = uc($site);
		$self->{ustream}->setSite($site);
		if ($self->{ustream}->siteExists()) {
			$self->add_message("ERROR: $site already exists");
			return(undef);
		}
		my $host = $self->{query}->param("ipaddress");
		my $port = $self->{query}->param("ipport");
		if (!defined($host) || !defined($port)) {
			$self->add_message("ERROR: no $host and/or $port defined");
			return(undef);
		}
		my $enable = $self->{query}->param("enablesite");
		my $ntripbinex = $self->{query}->param("enablebinex");
		my $ntriprtcm = $self->{query}->param("enablertcm");
		$self->{ustream}->createSite(	"site.ipaddr" => $host, "site.ipport" => $port,
					"site.enabled" => ($enable ne "") ? "TRUE": "FALSE",
					"ntrip.binex" => ($ntripbinex ne "") ? "TRUE": "FALSE",
					"ntrip.rtcm" => ($ntriprtcm ne "") ? "TRUE": "FALSE",
					"ntrip.rtcmv24" => ($ntriprtcm ne "") ? "TRUE": "FALSE",
					"ntrip.rtcmv30" => ($ntriprtcm ne "") ? "TRUE": "FALSE");
		$self->{ustream}->updateSystem();
		$self->{ustream}->getLogger()->info("config",getString({	type => "site",
							status => "create",
							site => $site,
							ipaddr => $host,
							ipport => $port
						}));
		$self->add_message("Site $site added to system");
		$self->mode_all();
	} elsif ($form->submitted eq "Cancel") {
		$self->add_message("Canceled: Site Addition canceled");
		$self->mode_all();
	} else {
		$self->add_content($form->render());
	}
}

=item UStream::CGI::mode_modify

Allow modification of site information throgh web interface

=cut

sub mode_modify {
	my $self = shift;
	my @sites = $self->{query}->param("site");
	my $site = $sites[0];
	$self->{ustream}->setSite($site);
	my %vars = $self->{ustream}->getSiteConfigHash();
	my $form = CGI::FormBuilder->new(
		params => $self->{query},           # params from CGI.pm 
		submit => [qw/Modify Cancel/],
		keepextras => 1             # keep mode param 
	);
	foreach (sort(keys(%vars))) {
		my $label = $_;
		my $value = $vars{$label};
		s/\./X2X/g;
		my $name = $_;
		$_ = $value;
		if (/true/i) {
			$form->field(name => "${name}_opt", label => "", options => ["$label"], value => $label);
		} elsif (/false/i) {
			$form->field(name => "${name}_opt", label => "", options => ["$label"]);
		} else {
			$form->field(name => $name, label => "$label", value => $value);
		}
	}

	if (($form->submitted eq "Modify") && $form->validate) {
		my @params = $form->field();
		my %vals;
		$self->add_content("<span id=\"modresults\">");
		foreach (@params) {
			next unless(/X2X/);
			my $val = $self->{query}->param($_);
			s/X2X/\./g;
			if (/_opt$/) {
				s/_opt$//g;
				if ($val eq "") {
					$val = "FALSE";
				} else {
					$val = "TRUE";
				}
			}
			$vals{$_} = $val;
			my $rc = $self->{ustream}->modifySiteConfig($_ => $val);
			if ($rc == 0) {
				$self->add_content("modify $_ => $val: Succeeded<p/>");
			} else {
				$self->add_message("modify $_ => $val: FAILED<p/>");
			}
		}
		$self->add_content("</span>");
		$self->{ustream}->writeSiteConfig();
		$self->{ustream}->updateSystem();
		$self->{ustream}->getLogger()->info("config",getString({   type => "site", status => "modify", site => $site }));
		$self->mode_all();
	} elsif ($form->submitted eq "Cancel") {
		$self->add_message("Canceled: Site modification canceled");
		$self->mode_all();
	} else {
		$self->add_content("<span id=\"modify\">");
		$self->add_content("<h2>Modify $site</h2><p/>");
		$self->add_content($form->render());
		$self->add_content("</span>");
	}
}

=item UStream::CGI::mode_delete

Delete site from system

=cut

sub mode_delete {
	my $self = shift;
	# New article
	my @sites = $self->{query}->param("site");
	my $site = $sites[0];
	my $form = CGI::FormBuilder->new(
		params => $self->{query},           # params from CGI.pm 
		submit => [qw/Delete Cancel/],
		keepextras => 1             # keep mode param 
	);
	$form->field(name => "deletesite", label => "Delete Site",  options => [$site]);
	if (($form->submitted eq "Delete") && $form->validate) {
		my $deletesite = $self->{query}->param("deletesite");
		if ($deletesite eq "") {
			$self->add_message("ERROR: Must check box to delete");
			$self->add_content($form->render());
			return(0);
		}

		my $site = $self->{query}->param("site");
		$site = uc($site);
		$self->{ustream}->setSite($site);
		if (! $self->{ustream}->siteExists()) {
			$self->add_message("ERROR: site $site does not exist, cannot delete");
			$self->mode_all();
			return(undef);
		}
		my $rc = $self->{ustream}->deleteSite();
		if ($rc == 0) {
			$self->add_message("DELETED: $site");
		} else {
			$self->add_message("ERROR: Could not delete $site");
		}
		$self->{ustream}->updateSystem();
		$self->{ustream}->getLogger()->info("config",getString({   type => "site", status => "delete", site => $site }));
		$self->mode_all();

	} elsif ($form->submitted eq "Cancel") {
		$self->add_message("CANCELED: Site deletion canceled");
		$self->mode_all();
	} else {
		$self->add_content($form->render());
	}
}

=item UStream::CGI::mode_log

Show log information for a site

=cut

sub mode_log {
	my $self = shift;
	# New article
	my @sites = $self->{query}->param("site");
	if ($#sites < 0) {
		$self->add_message("ERROR: no site passed");
		$self->mode_all();
		return(undef);
	}
	my $site = uc($sites[0]);
	$self->{ustream}->setSite($site);
	if (! $self->{ustream}->siteExists()) {
		$self->add_message("ERROR: Site $site doesn't exist");
		$self->mode_all();
		return(0);
	} else {
		my @logs = $self->{ustream}->getLogger()->getSiteEntries($site);
		my $logs = {};
		foreach (@logs) {
			my @hi = split(/\|/,$_);
			my ($act,$type) = split(":",$hi[2]);
			$logs->{$type} = [] unless (defined($logs->{$type}));
			push(@{$logs->{$type}}, $_);
		}
		$self->add_content("<span id=\"logs\">");
		$self->add_content("<h2>Logs for $site</h2>");
		$self->add_content("<span id=\"buttons\">");
		foreach my $type (sort(keys(%{$logs}))) {
			$self->add_content("<a href=\"#$type\">$type</a>");
		}
		$self->add_content("</span>");
			
		foreach my $type (sort(keys(%{$logs}))) {
			$self->add_content("<a name=\"$type\"><h3>$type</h3></a><p/>");
			foreach my $entry (reverse(@{$logs->{$type}})) {
				$self->add_content("$entry<p/>");
			}
		}
		$self->add_content("</span>");
	}
}

=item UStream::CGI::system_change

stop/start/restart entire UStream system after confirmation

=cut

sub system_change {
	my $self = shift;
	my $mode = shift;

	my $form = CGI::FormBuilder->new(
		params => $self->{query},           # params from CGI.pm 
		submit => [$mode, "Cancel"],
		keepextras => 1             # keep mode param 
	);
	my @opt;
	push(@opt,"$mode System");
	
	$form->field(name => "$mode", label => "",  options => \@opt);

	if (($form->submitted eq $mode) && $form->validate) {
		my $val = $self->{query}->param($mode);
		if ($val eq "") {
			$self->add_message("ERROR: Must check box to $mode system");
			$self->add_content($form->render());
			return(0);
		}
		
		my $com = $self->{ustream}->metaReplace("{path.bin}/ustreamrc $mode");
		my $rc = open(RC,"$com |");
		if (! $rc) {
			$self->add_message("ERROR: Unable to run ustreamrc $mode");
			$self->mode_all();
			return(undef);
		}
		$self->add_content("<pre>");
		while (<RC>) {
			$self->add_content($_);
		}
		$self->add_content("</pre>");
	} elsif ($form->submitted eq "Cancel") {
		$self->add_message("CANCELED: System $mode canceled");
		$self->mode_all();
	} else {
		$self->add_message("May take several minutes");
		$self->add_content($form->render());
	}
}

=item UStream::CGI::mode_info

load redirected info page for a specific site

=cut

sub mode_info {
	my $self = shift;

	my @sites = $self->{query}->param('site');
	if ($#sites != 0) {
		$self->add_message("ERROR: No site to show");
		$self->mode_all();
		return(0);
	}
	my $site = uc($sites[0]);
	$self->{ustream}->setSite($site);
	if (! $self->{ustream}->siteExists()) {
		$self->add_message("Site $site doesn't exist");
		return(0);
	} else {
		my $htmlfile = $self->{ustream}->metaReplace("{path.config}/site_info.html");
		if (! -f $htmlfile) {
			$self->{ustream}->add_message("Site Info file $htmlfile doesn't exist");
			return(0);
		}
		if (! open(HTML,"< $htmlfile")) {
			$self->{ustream}->add_message("Cannot open info file $htmlfile");
			return(0);
		}
		while (<HTML>) {
			$self->add_content($self->{ustream}->metaReplace($_));
		}
		close(HTML);
	}
}

=item UStream::CGI::mode_status

output UStream system status for all sites

=cut

sub mode_status {
	my $self = shift;

	my @test = $self->{query}->param();
	my $csv = 0;
	foreach (@test) {
		$csv = 1 if (/^csv$/);
	}

	$self->system_menu unless($csv);

	my $com = $self->{ustream}->metaReplace("{path.bin}/ustreamrc status");
	my $rc = open(RC,"$com |");
	if (! $rc) {
		$self->add_message("ERROR: Unable to run ustreamrc status");
		$self->mode_all();
		return(undef);
	}
	my %uptime;
	$self->add_content("<div id=\"status\"><h2>System Status</h2></div>") unless ($csv);
	while (<RC>) {
		my @hi = split();

		$hi[0] =~ s/:$//g;
		my $test = fileparse($hi[0]);
		$uptime{$test} = $hi[4];

	}
	$rc = close(RC);
	
	if ($csv) {
		$self->add_content("<pre>");
		my @sites = $self->{ustream}->getSiteList();
		foreach my $site (@sites) {
			my $summary = $self->makeSiteSummary($site, $uptime{$site});
			my @t = map { $summary->{$_}->{text} } sort(keys(%{$summary}));;
			my $t = join(",",@t);
			$self->add_content($t);
		}
		$self->add_content("</pre>");
		return(0);
	}
	$self->add_content("<table border=1 id=\"summary\">");
	$self->add_content("<tr id=\"Top\">");
	$self->add_content("<th rowspan=2><a href=\"/cgi-bin/ustream.cgi?mode=all\">Station</a></th>");
	$self->add_content("<th rowspan=2>Logs</th>");
	$self->add_content("<th rowspan=2>SiteInfo</th>");
	$self->add_content("<th colspan=5>Activated</th>");
	$self->add_content("<th rowspan=2>Uptime (seconds)</th>");
	$self->add_content("<th rowspan=2>Status</th>");
	$self->add_content("<th colspan=3>Latency (seconds)</th>");
	$self->add_content("<th colspan=2>Completeness (%)</th>");
	$self->add_content("<th colspan=3>Bandwidth (kbits/sec)</th>");
	$self->add_content("</tr>");
	$self->add_content("<tr>");
	$self->add_content("<th rowspan=1>Station</th>");
	$self->add_content("<th rowspan=1>RAW</th>");
	$self->add_content("<th rowspan=1>RTCM</th>");
	$self->add_content("<th rowspan=1>RTCM2.4</th>");
	$self->add_content("<th rowspan=1>RTCM3.0</th>");
	$self->add_content("<th rowspan=1>Latest</th>");
	$self->add_content("<th rowspan=1>1-hour mean</th>");
	$self->add_content("<th rowspan=1>1-day mean</th>");
	$self->add_content("<th rowspan=1>Last Hour</th>");
	$self->add_content("<th rowspan=1>Last Day</th>");
	$self->add_content("<th rowspan=1>Latest</th>");
	$self->add_content("<th rowspan=1>1-hour mean</th>");
	$self->add_content("<th rowspan=1>1-day mean</th>");
	$self->add_content("</tr>");
	my @sites = $self->{ustream}->getSiteList();
	foreach my $site (sort(@sites)) {
		my $id = "";
		my $summary = {};
		my $summary = $self->makeSiteSummary($site, $uptime{$site});
		if ($summary->{"03enable"}->{id} eq "Yes") {
			$self->add_content("<tr>");
		} else {
			$id = "Inactive";
			$self->add_content("<tr id=\"$id\">");
		}
		foreach my $sum (sort(keys(%{$summary}))) {
			my $td = "td";
			$td = "th" if ($sum =~ /^01Site/);
			my $id = $summary->{$sum}->{id} unless ($id eq "Inactive");
			my $url = $summary->{$sum}->{url};
			my $text = $summary->{$sum}->{text};
			if ($id ne "") {
				$self->add_content("<$td id=\"$id\">");
			} else {
				$self->add_content("<$td>");
			}
			if ($url ne "") {
				$self->add_content("<a href=\"$url\">");
			}
			$self->add_content($text);
			if ($url ne "") {
				$self->add_content("</a>");
			}
			$self->add_content("</$td>");
		}
		$self->add_content("</tr>");
	}
	$self->add_content("</table>");
	return(0);
}

=item UStream::CGI::mode_start

Start the UStream system

=cut

sub mode_start {
	my $self = shift;
	$self->system_menu();
	$self->system_change("start");
}

=item UStream::CGI::mode_stop

Stop the UStream system

=cut

sub mode_stop {
	my $self = shift;
	$self->system_menu();
	$self->system_change("stop");
}


=item UStream::CGI::mode_restart

Restart the UStream system

=cut

sub mode_restart {
	my $self = shift;
	$self->system_menu();
	$self->system_change("restart");
}


=item UStream::CGI::system_menu

add HTML code for the UStream system

=cut

sub system_menu {
	my $self=shift;

	$self->add_content("<span id=\"menu\">");
	$self->add_content("<a href=\"$self->{script_name}?mode=start\">Start System</a>");
	$self->add_content("<a href=\"$self->{script_name}?mode=stop\">Stop System</a>");
	$self->add_content("<a href=\"$self->{script_name}?mode=restart\">Restart System</a>");
	$self->add_content("<a href=\"$self->{script_name}?mode=status\">System Status</a>");
	$self->add_content("</span>");

}

=item UStream::CGI::mode_system

Print out system status

=cut

sub mode_system {
	my $self = shift;

	$self->mode_status();

	return(0);
}

=item UStream::CGI::mode_system

Get site information and status for a given site

=cut

sub makeSiteSummary {
	my $self = shift;
	my $site = shift;
	my $uptime = shift;

	my $summary = {};

	$summary->{"01Site"} = { id => "", url => "/cgi-bin/ustream.cgi?mode=show&site=$site", text => $site };
	$summary->{"02Log"} = { id => "", url => "/cgi-bin/ustream.cgi?mode=log&site=$site", text => "Logs" };
	$summary->{"02Z_Site"} = { id => "", url => "/cgi-bin/ustream.cgi?mode=info&site=$site", text => "Info" };

	my ($enable, $binex, $rtcm, $rtcm24, $rtcm30) = $self->getSiteStatus($site);
	$summary->{"03enable"} = { id => $enable, url => "", text => $enable};
	$summary->{"04raw"} = { id => $binex, url => "", text => $binex};
	$summary->{"05rtcm"} = { id => $rtcm, url => "", text => $rtcm};
	$summary->{"05rtcm24"} = { id => $rtcm24, url => "", text => $rtcm24};
	$summary->{"05rtcm30"} = { id => $rtcm30, url => "", text => $rtcm30};

	if ($enable eq "No") {
		$summary->{"07summary"} = { id => "Off", url => "", text => "Off"};
		return($summary);
	}
	$summary->{"06uptime"} = $self->makeSummaryEntry("Uptime", $uptime, "", $uptime);

	my $ret = $self->getLogStats($site, $summary);

	my $status = "OK";
	foreach (keys(%{$summary})) {
		next if (/1Day$/);
		$status = "Warn" if ($summary->{$_}->{id} eq "Warn");
		$status = "Fail" if ($summary->{$_}->{id} eq "Fail");
		last if ($status eq "Fail");
	}
	$summary->{"07summary"} = { id => $status, url => "", text => $status};

	return($summary);

}

=item UStream::CGI::getLogStats

Get the statiistics for a give site from the DB File

=cut

sub getLogStats {
	my $self = shift;
	my $site = shift;
	my $summary = shift;

	$self->{ustream}->setSite($site);
	return($summary) if (! $self->{ustream}->siteExists());

	$summary->{"08LatencyLatest"} = { id => "Fail", url => "", text => "No Data"};
	$summary->{"09Latency1Hour"}  = { id => "Fail", url => "", text => "No Data"};
	$summary->{"10Latency1Day"}   = { id => "Fail", url => "", text => "No Data"};

	$summary->{"12Complete1Hour"} = { id => "Fail", url => "", text => "No Data"};
	$summary->{"13Complete1Day"}  = { id => "Fail", url => "", text => "No Data"};

	$summary->{"14BaudLatest"}    = { id => "Fail", url => "", text => "No Data"};
	$summary->{"15Baud1Hour"}     = { id => "Fail", url => "", text => "No Data"};
	$summary->{"16Baud1Day"}      = { id => "Fail", url => "", text => "No Data"};

	my $logdir = $self->{ustream}->getDir("path.log");
	my $statdb = catfile($logdir,"${site}-stats");
	my %STATS;
	unless (tie(%STATS, 'GDBM_File', $statdb, O_RDONLY, 0666)) {
		return($summary);
	}

	my $hour_ago = time() - 3600;
	my $day_ago = time() - (24 * 3600);

#exampres (.1 = uptime, .2 = latency);
#key = 1185574560.1, value = uptime;61160;10488838;1371 (desc;seconds;bytes;baud)
#key = 1185574560.2, value = latency;1397;167752;120 (desc;avg_latency;total_latency;records)

	my @rev_keys = reverse(sort(keys(%STATS)));
	my $have_lat = 0;
	my $have_baud = 0;
	my $have_comp = 0;
	my @hour_lat;
	my @day_lat;
	my @hour_baud;
	my @day_baud;
	my $day_recs = 0;
	my $hour_recs = 0;
	foreach (@rev_keys) {
		last if ($_ < $day_ago);
		my @vals = split(';',$STATS{$_});
		if (/\.2$/) {
			push(@day_lat,$vals[1]) if ($_ > $day_ago);
			$day_recs += $vals[3] if ($_ > $day_ago);
			push(@hour_lat,$vals[1]) if ($_ > $hour_ago);
			$hour_recs += $vals[3] if ($_ > $hour_ago);
			next if ($_ < $hour_ago);
			if (! $have_lat ) {
				my $lat = ($vals[1] / 1000.0);
				$summary->{"08LatencyLatest"} = $self->makeSummaryEntry("Latency", $lat, "", sprintf("%.2f",$lat));
				$have_lat = 1;
			}
		}
		if (/\.1$/) {
			push(@day_baud,$vals[3]) if ($_ > $day_ago);
			push(@hour_baud,$vals[3]) if ($_ > $hour_ago);
			next if ($_ < $hour_ago);
			if (! $have_baud ) {
				my $bd = $vals[3];
				$summary->{"14BaudLatest"} = $self->makeSummaryEntry("Baud", $bd/1000.0, "", sprintf("%.2f",$bd/1000.0));
				$have_baud = 1;
			}
		}
	}
	if ($day_recs > 0) {
		my $percent = ($day_recs / 86400.0) * 100.0;
		$percent = 100 if ($percent > 100);
		$summary->{"13Complete1Day"} = $self->makeSummaryEntry("Completeness", $percent, "", sprintf ("%3d%%", $percent));
		$summary->{"13Complete1Day"}->{"id"} = "Warn" if ($summary->{"13Complete1Day"}->{"id"} eq "Fail");
	}
	if ($hour_recs > 0) {
		my $percent = ($hour_recs / 3600.0) * 100.0;
		$percent = 100 if ($percent > 100);
		$summary->{"12Complete1Hour"} = $self->makeSummaryEntry("Completeness", $percent, "", sprintf ("%3d%%", $percent));
	}
	if ($#hour_lat >= 0) {
		my $ave = $self->getAverage(@hour_lat);
		$summary->{"09Latency1Hour"} = $self->makeSummaryEntry("Latency", $ave/1000.0, "", sprintf ("%.2f", $ave/1000.0));
	}
	if ($#day_lat >= 0) {
		my $ave = $self->getAverage(@day_lat);
		$summary->{"10Latency1Day"} = $self->makeSummaryEntry("Latency", $ave/1000.0, "", sprintf ("%.2f", $ave/1000.0));
		$summary->{"10Latency1Day"}->{"id"} = "Warn" if ($summary->{"10Latency1Day"}->{"id"} eq "Fail");
	}
	if ($#hour_baud >= 0) {
		my $ave = $self->getAverage(@hour_baud);
		$summary->{"15Baud1Hour"} = $self->makeSummaryEntry("Baud", $ave/1000.0, "", sprintf ("%.2f", $ave/1000.0));
	}
	if ($#day_baud >= 0) {
		my $ave = $self->getAverage(@day_baud);
		$summary->{"16Baud1Day"} = $self->makeSummaryEntry("Baud", $ave/1000.0, "", sprintf ("%.2f", $ave/1000.0));
		$summary->{"16Baud1Day"}->{"id"} = "Warn" if ($summary->{"16Baud1Day"}->{"id"} eq "Fail");
	}

	my %vars = $self->{ustream}->getSiteConfigHash();
	my $dt = $vars{"site.datatype"};
	if ($dt eq "raw") {
		$summary->{"08LatencyLatest"} = { id => "Inactive", url => "", text => "N/A"};
		$summary->{"09Latency1Hour"}  = { id => "Inactive", url => "", text => "N/A"};
		$summary->{"10Latency1Day"}   = { id => "Inactive", url => "", text => "N/A"};

		$summary->{"12Complete1Hour"} = { id => "Inactive", url => "", text => "N/A"};
		$summary->{"13Complete1Day"}  = { id => "Inactive", url => "", text => "N/A"};

	}


	return($summary);
}

=item UStream::CGI::makeSummaryEntry

Set up site information summary entries with correct div values for display

=cut

sub makeSummaryEntry {
	my $self = shift;
	my $type = shift;
	my $value = shift;
	my $url = shift;
	my $text = shift;

	my $id = "OK";

	switch ($type) {

		case "Uptime" { 
				$id = "Warn" if ($value < 180);
				$id = "Fail" if ($value < 60);
				if (! defined($value) ) {
					$id = "Fail";
					$text = "Site Down?";
				}
				}
		case "Latency" {
				$id = "Warn" if ($value > 2.000);
				$id = "Fail" if ($value > 2.500);
				}
		case "Completeness" {
				$id = "Warn" if ($value < 98);
				$id = "Fail" if ($value < 95);
				}
		case "Baud" {
				$id = "Warn" if ($value > 2.000);
				$id = "Fail" if ($value > 2.200);
				}
	}
	return({ "id" => $id, "url" => $url, "text" => $text });
}

sub getAverage {
	my $self = shift;

	return(0) if ($#_ < 0);
	my $num = 0;
	foreach (@_) { $num += $_; }
	return ($num / ($#_ + 1));
}

=item UStream::CGI::getSiteStatus

Get information about a site status (enabled/disabled)

=cut

sub getSiteStatus {
	my $self = shift;
	my $site = shift;

	$self->{ustream}->setSite($site);
	return(undef) if (! $self->{ustream}->siteExists());

	my %vars = $self->{ustream}->{SITE_CFG}->vars();
	
	my $en = "No";
	my $bi = "No";
	my $rt = "No";
	my $rt24 = "No";
	my $rt30 = "No";
	$en   = "Yes" if ($vars{"site.enabled"} =~ /true/i);
	$bi   = "Yes" if ($vars{"ntrip.binex"} =~ /true/i);
	$rt   = "Yes" if ($vars{"ntrip.rtcm"} =~ /true/i);
	if ($rt eq "Yes") {
		$rt24 = "Yes" if ($vars{"ntrip.rtcmv24"} =~ /true/i);
		$rt30 = "Yes" if ($vars{"ntrip.rtcmv30"} =~ /true/i);
	}
	return($en, $bi, $rt, $rt24, $rt30);
}

1;

__END__

=back

=head1 AUTHOR

Keith F. Stark (Stark Consulting LLC)

=head1 SEE ALSO

L<ustreamrc>, L<ustream-logger>, L<ustream.cgi>, L<UStream>,  L<UStream::CGI>, L<UStream::Log>, L<UStream::Template>

=cut

