#!/usr/bin/perl

package UStream::Template;

=head1 NAME

UStream::Template - File Templates for run files and web files

=head1 SYNOPSIS

use UStream::Template;

=head1 DESCRIPTION

This packages defines a hash of files and their contents. These are written out after passing through the metareplace routine in UStream.

They are here in order to reduce installation clutter and allow for easy meta string replacement without having to read in the files.

=cut


use strict;

our %files;
our %contents;
$files{LOG} = "{path.config}/run.log";
$files{XINETD} = "{path.config}/run.xinetd";
$files{XINETD_CONF} = "{path.config}/xinetd.conf";
$files{XINETD_SITE} = "{path.config}/xinetd_site.conf";
$files{NTRIPC} = "{path.config}/run.ntripcaster";
$files{SHARC} = "{path.config}/run.sharc";
$files{HTML} = "{path.config}/ustream.html";
$files{CSS} = "{path.config}/ustream_style.css";
$files{HTTPD} = "{path.config}/ustream.httpd.conf";

=item UStream::Template->contents{LOG}

Run file template for logging service

=cut

$contents{LOG} ='#!/bin/bash

if [ $?PERLLIB ]; then PERLLIB= ; fi
export PERLLIB={base_path}/lib:$PERLLIB

cwd=`pwd`
site=$(basename $(dirname $cwd))

cd {path.log}

if [ -f {path.bin}/ustreamlogger ]; then
	exec {path.bin}/ustreamlogger $site > /dev/null 2>&1
fi

exec ustreamlogger $site > /dev/null 2>&1

';


=item UStream::Template->contents{XINETD}

Run file template for xinetd service

=cut
$contents{XINETD} ='#!/bin/bash

dir=`pwd`

if [ ! -f running ]; then
	touch running;
else
	perl -e \'my $t = ((stat("running"))[9] - (time()-30)); sleep($t) if ($t>0); \'
	touch running;
fi

OPTIONS="-dontfork -stayalive -f $dir/xinetd.conf -pidfile $dir/xinetd.pid"

if [ -f /sbin/xinetd ]; then
	exec /sbin/xinetd $OPTIONS
fi

if [ -f /usr/sbin/xinetd ]; then
	exec /usr/sbin/xinetd $OPTIONS
fi

if [ -f /usr/local/sbin/xinetd ]; then
	exec /usr/local/sbin/xinetd $OPTIONS
fi

if [ -f /opt/sbin/xinetd ]; then
	exec /opt/sbin/xinetd $OPTIONS
fi

';

=item UStream::Template->contents{NTRIPC}

Run file template for ntripcaster service

=cut
$contents{NTRIPC} ='#!/bin/bash

if [ ! -f running ]; then
	touch running;
else
	perl -e \'my $t = ((stat("running"))[9] - (time()-30)); sleep($t) if ($t>0); \'
	touch running;
fi

if [ -f {path.bin}/ntripcaster ]; then
	exec {path.bin}/ntripcaster
fi

if [ -f {path.bin}/ntripdaemon ]; then
	exec {path.bin}/ntripdaemon
fi

if [ -f /usr/sbin/ntripdaemon ]; then
	exec /usr/sbin/ntripdaemon
fi

if [ -f `which ntripdaemon` ]; then
	exec `which ntripdaemon`
fi

';

=item UStream::Template->contents{SHARC}

Run file template for sharc service

=cut
$contents{SHARC} ='#!/bin/bash

dir=`pwd`

if [ ! -f running ]; then
	touch running;
else
	perl -e \'my $t = ((stat("running"))[9] - (time()-30)); sleep($t) if ($t>0); \'
	touch running;
fi

cd {path.data}

site=`basename $dir`
echo "$site:connect: status=startup"

if [ -f ./sharc ]; then
	exec ./sharc --configfile $dir/sharc.cfg
fi

if [ -f {path.bin}/sharc.bat ]; then
	exec {path.bin}/sharc.bat --configfile $dir/sharc.cfg
fi

if [ -f {path.bin}/sharc ]; then
	exec {path.bin}/sharc --configfile $dir/sharc.cfg
fi

exec sharc --configfile $dir/sharc.cfg 

';

=item UStream::Template->contents{XINETD_CONF}

configuration file template for xinetd.conf

=cut
$contents{XINETD_CONF} ='
defaults
{
	cps		= 50 10
	instances	= 20
	per_source	= 3
	v6only		= no
	groups		= yes
	umask		= 002
}

includedir {base_path}/xinetd.d
';

=item UStream::Template->contents{XINETD_SITE}

Site configuration file template for xinetd

=cut
$contents{XINETD_SITE} ='
service {site.code}_raw
{
        port                    = 9{site.refstaid}
        type                    = UNLISTED
        socket_type             = stream
        wait                    = no
        user                    = {ustream_user}
        server                  = {path.bin}/ntripclient
        server_args             = -h {server1.host} -p {server1.port} -d {site.code}
        disable                 = {xinetd.raw}
}

service {site.code}_rtcm24
{
        port                    = 8{site.refstaid}
        type                    = UNLISTED
        socket_type             = stream
        wait                    = no
        user                    = {ustream_user}
        server                  = {path.bin}/ntripclient
        server_args             = -h {server1.host} -p {server1.port} -d {site.code}_RTCM
        disable                 = {xinetd.rtcm24}
}

service {site.code}_rtcm30
{
        port                    = 7{site.refstaid}
        type                    = UNLISTED
        socket_type             = stream
        wait                    = no
        user                    = {ustream_user}
        server                  = {path.bin}/ntripclient
        server_args             = -h {server1.host} -p {server1.port} -d {site.code}_RTCM3
        disable                 = {xinetd.rtcm30}
}
';

=item UStream::Template->contents{HTML}

ustream.html template

=cut
$contents{HTML} ='<?xml version="1.0" encoding="utf-8"?>
<!DOCTYPE html PUBLIC "-//W3C//DTD XHTML 1.0 Strict//EN" "http://www.w3.org/TR/xhtml1/DTD/xhtml1-strict.dtd">
<html xmlns="http://www.w3.org/1999/xhtml" xml:lang="en" lang="en">

<html>
        <head>
                <title>
                        Redirecting you to UStream...
                </title>
                <meta http-equiv="Refresh" content="1;URL=/cgi-bin/ustream.cgi">
        </head>
        <body>
                Redirecting you to UStream...
        </body>
</html>
';

=item UStream::Template->contents{HTTPD}

Example HTTPD configuration file.

=cut
$contents{HTTPD} ='
## This is a framework file only
## apache should be configured to run as the ustream user
## and this file should be copied to /etc/httpd/conf.d

<Directory "{base_path}/www">
    AllowOverride AuthConfig
    AddHandler cgi-script cgi
</Directory>

<Directory "{base_path}/www/html">
    Options Indexes FollowSymLinks MultiViews Includes ExecCGI
    AllowOverride All
    Order allow,deny
    Allow from all
</Directory>

<Directory "{base_path}/www/cgi-bin">
    Options Indexes FollowSymLinks MultiViews Includes ExecCGI
    AllowOverride All
    Order allow,deny
    Allow from all
</Directory>

<VirtualHost *:80>
    ServerAdmin ustream@example.com
    DocumentRoot {base_path}/www/html
    ServerName ustream.example.com
    ErrorLog {base_path}/logs/apache/error_log
    CustomLog {base_path}/logs/apache/access_log common
</VirtualHost>
';

=item UStream::Template->contents{CSS}

CSS File for displaying ustream.cgi output with colors

=cut
$contents{CSS} ='

body {font: .7em Verdana, sans-serif; background: #F8F8FF;}

#menu a {
        font-size:15pt;
        vertical-align: bottom;
        border-right: 1px solid gray;
        border-left: 1px solid gray;
        border-top: 1px solid gray;
        border-bottom: 1px solid gray;
        padding: 0em 1em 0em 1em;
}

#buttons a {
        font-size:8pt;
        border-right: 1px solid gray;
        border-left: 1px solid gray;
        border-top: 1px solid gray;
        border-bottom: 1px solid gray;
}

#error {
        font-size:14pt;
        color:red;
}

#sitelist {
        font-size:17pt;
        font-weight:bold;
        padding: 1em 1em 1em 1em;
}

#sitesettings {
        font-size:12;
}

#sitesettings th {
        padding-top:1em;
        font-size:15pt;
}

#header {
        border-bottom: 1px solid gray;
        padding-bottom:0px;
}

#logo {
        vertical-align:top;
        font-size:14pt;
}

#logs {
        font-size:12pt;
        border-right: 1px solid gray;
        border-left: 1px solid gray;
        border-top: 1px solid gray;
        border-bottom: 1px solid gray;
        padding: 0em 1em 0em 1em;
}

#summary #Yes {
	background-color: #CCFFCC;
}

#summary #Fail {
	background-color: #DD5050;
}

#summary #Warn {
	background-color: #FFCC00;
}

#summary #Top {
	background-color: #FFCC99;
}

#summary #Inactive {
	background-color: #9D9D9D;
}

#summary #No {
	background-color: #9D9D9D;
}
';

1;

__END__

=head1 AUTHOR

Keith F. Stark (Stark Consulting LLC)

=head1 SEE ALSO

L<ustreamrc>, L<ustream-logger>, L<ustream.cgi>, L<UStream>,  L<UStream::CGI>, L<UStream::Log>, L<UStream::Template>

=cut
